<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<!--
	jfc.xslt
	Copyright 2007-2009 Teppefall. All rights reserved. http://www.teppefall.com
	Author: Jan Erik Paulsen
 -->

<xsl:template match="processing-instruction()">
	<xsl:choose>
		<xsl:when test="local-name() = 'xsl-stylesheet'">
		    <xsl:comment> XSLT </xsl:comment>
		</xsl:when>
		<xsl:when test="local-name() = 'park'">
		    <void method="parkInstruction">
		    	<string><xsl:value-of select="."/></string>
	    	</void>
		</xsl:when>
		<xsl:when test="local-name() = 'park-bind'">
		    <void method="parkBinder">
		    	<string><xsl:value-of select="."/></string>
	    	</void>
		</xsl:when>
		<xsl:when test="local-name() = 'park-parameter'">
		    <void method="parkParameter">
		    	<string><xsl:value-of select="."/></string>
	    	</void>
		</xsl:when>
		<xsl:otherwise>
			<xsl:comment> Ignored PI <xsl:value-of select="local-name()"/></xsl:comment>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template name="parkComponent">
    <object idref="self">
        <void property="title"><string><xsl:value-of select="@title"/></string></void>
    </object>
    <xsl:if test="@height != ''">
        <object idref="self">
            <void method="setSize"><int><xsl:value-of select="@height"/></int><int><xsl:value-of select="@width"/></int></void>
        </object>
    </xsl:if>
    <xsl:if test="@icon != ''">
        <void property="icon"><string><xsl:value-of select="@icon"/></string></void>
    </xsl:if>
</xsl:template>

<xsl:template name="properties">
    <xsl:if test="@action != ''">
	    <void property="action">
	    	<object idref="{@action}"/>
<!--	    	<object idref="{@value}"/>-->
	    </void>
    </xsl:if>
    <xsl:if test="@focus != ''"><void method="requestFocusInWindow"/></xsl:if>
    <xsl:if test="@text != ''"><void property="text"><string><xsl:value-of select="@text"/></string></void></xsl:if>
    <xsl:if test="@name != ''"><void property="name"><string><xsl:value-of select="@name"/></string></void></xsl:if>
    <xsl:if test="@enabled = 'false'"><void property="enabled"><boolean>false</boolean></void></xsl:if>
    <xsl:if test="@mnemonic != ''"><void property="mnemonic"><char><xsl:value-of select="@mnemonic"/></char></void></xsl:if>
    <xsl:if test="@icon != ''">
	    <void property="icon">
	        <object class="javax.swing.ImageIcon">
	            <object class="java.net.URL">
	            <xsl:choose>
	            <xsl:when test="starts-with(@icon, 'http:')">
	                <string><xsl:value-of select="@icon"/></string>
	            </xsl:when>
	            <xsl:when test="starts-with(@icon, '/')">
	                <string><xsl:value-of select="$base"/><xsl:value-of select="substring(@icon, 2)"/></string>
	            </xsl:when>
	            <xsl:otherwise>
	                <string><xsl:value-of select="$base"/><xsl:value-of select="@icon"/></string>
	            </xsl:otherwise>
	            </xsl:choose>
	            </object>
	        </object>
	        <void property="borderPainted">
	            <boolean>false</boolean>
	        </void>
        </void>
    </xsl:if>
</xsl:template>

<xsl:template match="status">
	<object idref="self">
		<void property="status">
			<string><xsl:value-of select="@value"/></string>
		</void>
	</object>
</xsl:template>

<xsl:template match="action | object">
	<xsl:variable name="nameOrId"><xsl:value-of select="@name"/></xsl:variable>
	<xsl:if test="@name = ''"><xsl:variable name="nameOrId"><xsl:value-of select="@id"/></xsl:variable></xsl:if>
	<xsl:comment><xsl:value-of select="$nameOrId"/></xsl:comment>

	<object id="{$nameOrId}" class="{@class}">
		<xsl:if test="name() = 'action'">
			<void method="putValue">
				<string>Name</string>
				<string><xsl:value-of select="@text"/></string>
			</void>
		</xsl:if>
	</object>
</xsl:template>

<xsl:template match="idref" name="idref">
	<xsl:variable name="nameOrId"><xsl:value-of select="@name"/></xsl:variable>
	<xsl:if test="@name = ''"><xsl:variable name="nameOrId"><xsl:value-of select="@id"/></xsl:variable></xsl:if>
	<xsl:comment><xsl:value-of select="$nameOrId"/></xsl:comment>
	<object idref="{$nameOrId}"/>
</xsl:template>

<xsl:template match="listener">
	<xsl:choose>
		<xsl:when test="@type = 'caret'">
			<object idref="{@source}">
				<void method="addCaretListener">
					<object class="com.teppefall.ds.layout.jxml.AssemblyCaretListener">
						<object idref="{@source}"/>
						<object idref="{@button}"/>
					</object>
				</void>
			</object>
		</xsl:when>
		<xsl:when test="@type = 'undo'">
			<object class="com.teppefall.ds.AlterUndoRedo">
				<void method="create">
					<object idref="{@source}"/>
					<object idref="{@undo}"/>
					<object idref="{@redo}"/>
				</void>
			</object>
		</xsl:when>
		<xsl:when test="@type = 'selected'">
			<object idref="{@source}">
				<void method="addListSelectionListener">
					<object class="com.teppefall.ds.layout.jxml.AssemblyListItemSelectedListener">
						<object idref="{@button}"/>
					</object>
				</void>
			</object>
		</xsl:when>
		<xsl:when test="@type = 'visible'">
			<object idref="{@source}">
				<void method="addActionListener">
					<object class="com.teppefall.ds.layout.jxml.AssemblyVisible">
						<object idref="{@target}"/>
					</object>
				</void>
			</object>
		</xsl:when>
		<xsl:otherwise>
			<xsl:comment>
				Warning: Unknown listener !
			</xsl:comment>
		</xsl:otherwise>
	</xsl:choose>
	
</xsl:template>

<xsl:template match="component">
	<xsl:param name="componentClass">
		<xsl:choose>
			<xsl:when test="not(@class)">javax.swing.JPanel</xsl:when>
			<xsl:otherwise><xsl:value-of select="@class"/></xsl:otherwise>
		</xsl:choose>
	</xsl:param>
	<xsl:param name="componentId">
		<xsl:choose>
			<xsl:when test="not(@id)"></xsl:when>
			<xsl:otherwise><xsl:value-of select="@id"/></xsl:otherwise>
		</xsl:choose>
	</xsl:param>

<!--
	<xsl:comment>
		<xsl:value-of select="$componentClass"/>
		<xsl:value-of select="$componentId"/>
		<xsl:value-of select="xmlns:mx"/>
	</xsl:comment>
	-->

    <void method="add">
        <object class="{$componentClass}">
        <!-- id="{$componentId}"-->
        
        	<xsl:if test="@id">
        		<xsl:attribute name="id">
					<xsl:value-of select="$componentId"/>
        		</xsl:attribute>
        	</xsl:if>

        	<!--
        	<void method="putClientProperty">
        		<string>park.transformer</string>
        		<string><xsl:value-of select="$transformer"/></string>
        	</void>
        	<void method="putClientProperty">
        		<string>park.url</string>
        		<string><xsl:value-of select="$url"/></string>
        	</void>
        	-->
		    <xsl:if test="@binder != ''">
            	<void method="putClientProperty">
            		<string>park.x-jfcpanel</string>
				    <object idref="self"/>
            	</void>
	    	</xsl:if>
			<xsl:call-template name="properties"/>
			<xsl:call-template name="componentProperties"/>
		    <xsl:apply-templates/>
        </object>
	    <xsl:if test="@constraint != '' and name(parent::*) != 'TESTTESTlayout'">
	    	<!--<xsl:comment> <xsl:value-of select="name(parent::*)"/> </xsl:comment>-->
			<string><xsl:value-of select="@constraint"/></string>
	    </xsl:if>
    </void>
    <xsl:if test="@id != ''">
	    <object idref="self">
	        <void method="register">
		    	<string><xsl:value-of select="@id"/></string>
		    	<object idref="{@id}"/>
	        </void>
	    </object>
	</xsl:if>
    <xsl:for-each select="attribute">
	    <void method="setAttribute">
	    	<string><xsl:value-of select="@name"/></string>
	    	<string><xsl:value-of select="@value"/></string><!-- sjekk etter cdata i barn !! -->
	    </void>
    </xsl:for-each>
</xsl:template>


<xsl:template match="clientproperty">
	<void method="putClientProperty">
		<string><xsl:value-of select="@name"/></string>
		<xsl:choose>
			<xsl:when test="not(@type)">
				<string><xsl:value-of select="@value"/></string>
			</xsl:when>
			<xsl:when test="@type = 'Boolean'">
	       		<object class="java.lang.Boolean">
	       			<boolean><xsl:value-of select="@value"/></boolean>
	       		</object>
			</xsl:when>
			<xsl:when test="@type = 'Double'">
	       		<object class="java.lang.Double">
	       			<double><xsl:value-of select="@value"/></double>
	       		</object>
			</xsl:when>
			<xsl:otherwise>
				<xsl:comment> NOT IMPLEMENTED </xsl:comment>
				<!--<xsl:call-template name="componentProperties"/>-->
			</xsl:otherwise>
		</xsl:choose>
    </void>
</xsl:template>


<!--
egen fil
<library name="hei"><action name="cut" class=""/></library>
annen fil
<import href="hei.jfc"/>
  -->

<xsl:template match="library"/>
<xsl:template match="import"/>
<xsl:template match="property"/>

<xsl:template name="componentProperties">
    <xsl:for-each select="property">
		<void property="{@name}">
            <xsl:choose>
	            <xsl:when test="@type = 'boolean'"><boolean><xsl:value-of select="@value"/></boolean></xsl:when>
	            <xsl:when test="@type = 'byte'"><byte><xsl:value-of select="@value"/></byte></xsl:when>
	            <xsl:when test="@type = 'char'"><char><xsl:value-of select="@value"/></char></xsl:when>
	            <xsl:when test="@type = 'short'"><short><xsl:value-of select="@value"/></short></xsl:when>
	            <xsl:when test="@type = 'double'"><double><xsl:value-of select="@value"/></double></xsl:when>
	            <xsl:when test="@type = 'float'"><float><xsl:value-of select="@value"/></float></xsl:when>
	            <xsl:when test="@type = 'int'"><int><xsl:value-of select="@value"/></int></xsl:when>
	            <xsl:when test="@type = 'long'"><long><xsl:value-of select="@value"/></long></xsl:when>
	            <xsl:when test="@type = 'string'"><string><xsl:value-of select="@value"/></string></xsl:when>
	            <xsl:when test="@type = 'hashtable' or @type = 'vector' or @type = 'Hashtable' or @type = 'Vector'">
				    <xsl:apply-templates/>
	            </xsl:when>
	            <xsl:when test="@type = 'object' or @type = 'Object'">
		            <xsl:apply-templates/>
	            </xsl:when>
	            <xsl:when test="@type = 'class' or @type = 'class'">
	            	<object class="{@value}"/>
	            </xsl:when>
	            <xsl:when test="@type = 'idref' or @type = 'IDRef'">
	            	<object idref="{@value}"/>
	            </xsl:when>
	            <xsl:when test="@type = 'component' or @type = 'Component'">

					<!-- BIG UGLY HACK START -->
					
				    <xsl:for-each select="component">
				        <object class="{@class}" id="{@id}">
							<xsl:call-template name="properties"/>
				            <xsl:call-template name="componentProperties"/>
						    <xsl:if test="@id != ''">
							    <object idref="self">
							        <void method="register">
								    	<string><xsl:value-of select="@id"/></string>
								    	<object idref="{@id}"/>
							        </void>
							    </object>
							</xsl:if>
				            <xsl:apply-templates/>
					    </object>
				    </xsl:for-each>
				    <xsl:for-each select="layout">
						<xsl:variable name="unique">
							<xsl:if test="not(@id)">
							<xsl:value-of select="generate-id(.)"/>
							</xsl:if>
							<xsl:value-of select="@id"/>
						</xsl:variable>
						<object class="javax.swing.JPanel" id="{$unique}">
							<void property="opaque">
								<boolean>false</boolean>
							</void>
							<void property="layout">
								<object class="{@class}">
									<xsl:if test="@link">
										<object idref="{$unique}"/>
									</xsl:if>
						           <!--<xsl:call-template name="componentProperties"/>-->
								</object>
							</void>
							<!--  df -->
							<xsl:call-template name="properties"/>
				            <xsl:call-template name="componentProperties"/>
						    <xsl:apply-templates/>
						</object>
						<xsl:if test="@id != ''">
						    <object idref="self">
						        <void method="register">
							    	<string><xsl:value-of select="@id"/></string>
							    	<object idref="{@id}"/>
						        </void>
						    </object>
						</xsl:if>
				    </xsl:for-each>

					<!-- BIG UGLY STOP -->

				    <!--  target can not be null ?? 
		            <xsl:apply-templates/>-->

	            </xsl:when>
	            <xsl:when test="@type = 'color' or @type = 'Color'">
					<xsl:choose>
			            <xsl:when test="substring(@value, 1, 1) = '#'">
							<object class="java.awt.Color" method="decode">
					            <string><xsl:value-of select="@value"/></string>
				            </object>
			            </xsl:when>
			            <xsl:otherwise>
							<object class="java.awt.Color">
					            <int><xsl:value-of select="@value"/></int>
				            </object>
			            </xsl:otherwise>
					</xsl:choose>
	            </xsl:when>
	            <xsl:when test="@type = 'font' or @type = 'Font'">
					<object class="com.teppefall.ds.layout.jxml.AssemblyFont">
						<object idref="{@deriveFrom}"/>
						<int><xsl:value-of select="@value"/></int>
		            </object>
	            </xsl:when>
	            <xsl:when test="@type = 'insets' or @type = 'Insets'">
		            <xsl:if test="contains(@value, ',')">
						<object class="java.awt.Insets">
							<int><xsl:value-of select="substring-before(@value,',')"/></int>
							<int><xsl:value-of select="substring-after(@value,',')"/></int>
							<int><xsl:value-of select="substring-before(@value,',')"/></int>
							<int><xsl:value-of select="substring-after(@value,',')"/></int>
						</object>
					</xsl:if>
	            </xsl:when>
	            <xsl:when test="@type = 'dimension' or @type = 'Dimension'">
		            <xsl:if test="contains(@value, ',')">
						<object class="java.awt.Dimension">
							<int><xsl:value-of select="substring-before(@value,',')"/></int>
							<int><xsl:value-of select="substring-after(@value,',')"/></int>
						</object>
					</xsl:if>
	            </xsl:when>
<!--	            <xsl:when test="@type = 'rectangle' or @type = 'Rectangle'">
					<object class="java.awt.Rectangle">
						<int>0</int><int>0</int><int>200</int><int>200</int>
					</object>
	            </xsl:when>-->
	            <xsl:when test="@type = 'point' or @type = 'Point'">
		            <xsl:if test="contains(@value, ',')">
						<object class="java.awt.Point">
							<int><xsl:value-of select="substring-before(@value,',')"/></int>
							<int><xsl:value-of select="substring-after(@value,',')"/></int>
						</object>
					</xsl:if>
	            </xsl:when>
	            <xsl:when test="@type = 'icon' or @type = 'Icon'">
					<object class="com.teppefall.ds.layout.jxml.AssemblyIcon">
						<string><xsl:value-of select="@value"/></string>
					</object>
	            </xsl:when>
	            <xsl:when test="@type = 'tree' or @type = 'Tree'">
	            
					<object class="com.teppefall.ds.layout.jxml.AssemblyListModel">
					    <xsl:apply-templates/>
					</object>
	            </xsl:when>
	            <xsl:when test="@value != ''">
					<string><xsl:value-of select="@value"/></string>
	            </xsl:when>
	            <xsl:when test="not(@value)">
					<string><xsl:value-of select="."/></string>
	            </xsl:when>
	            <xsl:otherwise>
				    <xsl:apply-templates/>
	            </xsl:otherwise>
            </xsl:choose>
		</void>
    </xsl:for-each>
</xsl:template>

<!--
	See layout.
-->
<xsl:template match="gridlayout">
	<xsl:param name="unique">
		<xsl:if test="not(@id)">
		<xsl:value-of select="generate-id(.)"/>
		</xsl:if>
		<xsl:value-of select="@id"/>
	</xsl:param>
	<xsl:param name="columns">
		<xsl:choose>
		<xsl:when test="@columns != ''"><xsl:value-of select="@columns"/></xsl:when>
		<xsl:otherwise> 2 </xsl:otherwise>
		</xsl:choose>
	</xsl:param>
	<xsl:param name="rows">
		<xsl:choose>
		<xsl:when test="@rows != ''"><xsl:value-of select="@rows"/></xsl:when>
		<xsl:otherwise> 2 </xsl:otherwise>
		</xsl:choose>
	</xsl:param>
	<void method="add">
		<object class="javax.swing.JPanel" id="{$unique}">
			<void property="opaque">
				<boolean>false</boolean>
			</void>
			<void property="layout">
				<object class="java.awt.GridLayout">
					<int><xsl:value-of select="$rows"/></int>
					<int><xsl:value-of select="$columns"/></int>
					<int>1</int>
					<int>1</int>
		    		<xsl:if test="not(@resolve)">
	            	<xsl:call-template name="componentProperties"/> 
		            </xsl:if>
				</object>
			</void>
		    <xsl:if test="@resolve = 'component'">
	    	<xsl:call-template name="componentProperties"/>
		    </xsl:if>
		    <xsl:apply-templates/>
		</object>
	    <xsl:if test="@constraint != ''">
			<string><xsl:value-of select="@constraint"/></string>
	    </xsl:if>
	</void>
    <xsl:call-template name="register"/>
</xsl:template>

<!--
	Attribute @resolve set to "component" follows the same rules as component.
	The default is to set layout properties, not component properties.
 -->
<xsl:template match="layout">
	<xsl:param name="unique">
		<xsl:if test="not(@id)">
		<xsl:value-of select="generate-id(.)"/>
		</xsl:if>
		<xsl:value-of select="@id"/>
	</xsl:param>
	<!--
	<xsl:param name="host">
		<xsl:if test="not(@host)">
		javax.swing.JPanel
		</xsl:if>
		<xsl:value-of select="@host"/>
	</xsl:param>
	-->
	<void method="add">
		<object class="javax.swing.JPanel" id="{$unique}">
			<void property="opaque">
				<boolean>false</boolean>
			</void>
			<void property="layout">
				<object class="{@class}">
					<xsl:if test="@link">
						<object idref="{$unique}"/>
					</xsl:if>
            		<!--TODO logical ? -->
		    		<xsl:if test="not(@resolve)">
	            	<xsl:call-template name="componentProperties"/> 
		            </xsl:if>
				</object>
			</void>
            <!--TODO logical ? -->
		    <xsl:if test="@resolve = 'component'">
	    	<xsl:call-template name="componentProperties"/>
		    </xsl:if>
		    <xsl:apply-templates/>
		</object>
	    <xsl:if test="@constraint != ''">
			<string><xsl:value-of select="@constraint"/></string>
	    </xsl:if>
	</void>
    <xsl:if test="@id != ''">
	    <object idref="self">
	        <void method="register">
		    	<string><xsl:value-of select="@id"/></string>
		    	<object idref="{@id}"/>
	        </void>
	    </object>
	</xsl:if>
</xsl:template>

<xsl:template name="register">
    <xsl:if test="@id != ''">
	    <object idref="self">
	        <void method="register">
		    	<string><xsl:value-of select="@id"/></string>
		    	<object idref="{@id}"/>
	        </void>
	    </object>
	</xsl:if>
</xsl:template>

<xsl:template match="glue | filler | rigid">
	<void method="add">
		<xsl:choose>
			<xsl:when test="local-name() = 'glue'">
				<xsl:choose>
					<xsl:when test="@type='vertical'">
					    <object class="javax.swing.Box" method="createVerticalGlue"/>
					</xsl:when>
					<xsl:otherwise>
					    <object class="javax.swing.Box" method="createHorizontalGlue"/>
					</xsl:otherwise>
				</xsl:choose>
			</xsl:when>
			<xsl:when test="local-name() = 'filler'">
			    <!--<object class="javax.swing.Box$Filler">
					<object class="java.awt.Dimension">
						<int><xsl:value-of select="substring-before(@minimum,',')"/></int>
						<int><xsl:value-of select="substring-after(@minimum,',')"/></int>
					</object>
					<object class="java.awt.Dimension">
						<int><xsl:value-of select="substring-before(@preferred,',')"/></int>
						<int><xsl:value-of select="substring-after(@preferred,',')"/></int>
					</object>
					<object class="java.awt.Dimension">
						<int><xsl:value-of select="substring-before(@maximum,',')"/></int>
						<int><xsl:value-of select="substring-after(@maximum,',')"/></int>
					</object>
			    </object>-->
			</xsl:when>
			<xsl:when test="local-name() = 'rigid'">
			    <object class="javax.swing.Box" method="createRigidArea">
					<object class="java.awt.Dimension">
						<int><xsl:value-of select="substring-before(@value,',')"/></int>
						<int><xsl:value-of select="substring-after(@value,',')"/></int>
					</object>
			    </object>
			</xsl:when>
			<xsl:otherwise>
				<xsl:comment>
					Unknown: <xsl:value-of select="local-name()"/>
				</xsl:comment>
			</xsl:otherwise>
		</xsl:choose>
	</void>
</xsl:template>

<xsl:template match="group">
    <object class="javax.swing.ButtonGroup">
    	<xsl:for-each select="idref">
			<void method="add">
				<object idref="{@id}"/>
			</void>
		</xsl:for-each>
    </object>
</xsl:template>

<xsl:template match="focus">
	<void method="requestFocusInWindow"/><!-- Maa skje etter oppkobling -->
</xsl:template>


<xsl:template match="border">
    <object class="javax.swing.BorderFactory" method="createTitledBorder">
    	<string><xsl:value-of select="@title"/></string>
    </object>
</xsl:template>

<xsl:template match="border2">
    <object class="com.teppefall.ds.layout.jxml.AssemblyBorder" method="createBottomBorder"/>
</xsl:template>

<xsl:template match="emptyborder">
	<void property="border">
    <object class="javax.swing.BorderFactory" method="createEmptyBorder"/>
    </void>
</xsl:template>


<xsl:template match="binding">
    <object class="javax.beans.binding.Binding">
		<object idref="{@object}"/>
		<string><xsl:value-of select="@logic"/></string>
		<object idref="{@component}"/>
		<string><xsl:value-of select="@property"/></string>
<!--		<void method="putParameter">
		</void>-->
    </object>
</xsl:template>

<xsl:template match="script">
	<xsl:comment>
	<xsl:value-of select="."/>
	</xsl:comment>
<!--    <void method="addScript">
	    <string><xsl:value-of select="."/></string>
    </void>-->
</xsl:template>

<xsl:template name="addBrowser" match="browser">
	<void method="add">
		<object class="com.teppefall.massive.MassivePanel">
			<void method="browse">
				<string><xsl:value-of select="@href"/></string>
			</void>
		</object>
	</void>
</xsl:template>

</xsl:stylesheet>